const axios = require('axios');
const crypto = require('crypto');

function FqOpenApi(options) {
  if (!(this instanceof FqOpenApi)) {
    return new FqOpenApi(options);
  }
  const { appId, appSecret, isDingVersion } = options || {};
  this.appId = appId;
  this.appSecret = appSecret;
  // 网页版本和钉钉版本所需的域名不同
  this.host = isDingVersion ? 'https://ding.fanqier.cn/' : "https://fanqier.cn/";
}

/**
 * 获取校验TOKEN
 */
FqOpenApi.prototype.getToken = function() {
  const appid = this.appId;
  const appSecret = this.appSecret;
  if (!appid || !appSecret) {
    throw Error('invail appid or appSecret!');
  }
  const expired = 1800000;
  const sha1Code = crypto.createHash("sha1");
  const tokenJson = {
    appid,
    now: Date.now(),
    expired: Date.now() + expired
  };
  const str = JSON.stringify(tokenJson);
  const token = new Buffer(str).toString('base64') + '.' + sha1Code.update(str + appSecret, 'utf-8').digest("hex");
  return token;
}

/**
 * 获取表单列表
 * page: 分页：当前页，从零开始
 * pagesize: 分页：每页条数，默认20
 */
FqOpenApi.prototype.getFormList = async function(page = 0, pagesize = 20) {
  const token = this.getToken();
  return await axios.get(`${this.host}openapi/forms?token=${token}&page=${page}&pagesize=${pagesize}`).then(res => res.data);
}

/**
 * 获取表单结构
 * @params {String} formid 表单id
 */
FqOpenApi.prototype.getFormStruct = async function(formid) {
  if (!formid) {
    throw Error('invail id!');
  }
  const token = this.getToken();
  return await axios.get(`${this.host}openapi/forms/${formid}?token=${token}`).then(res => res.data);
}

/**
 * 获取表单
 * @params {Object} options {
 *  start: Number, // 时间过滤：开始时间，毫秒值，可选
 *  end: Number, // 时间过滤：结束时间，毫秒值，可选
 *  page: Number, // 分页当前页 从0开始
 *  pagesize: Number // 分页的每页数量 默认1000 最大不超过5000
 * }
 */
FqOpenApi.prototype.getFormDatas = async function(formid, options) {
  if (!formid) {
    throw Error('invail id!');
  }
  const token = this.getToken();
  const params = { token };
  if (options) {
    Object.assign(params, options);
  }
  return await axios.get(`${this.host}openapi/forms/${formid}/datas`, { params }).then(res => res.data);
}

module.exports = FqOpenApi;
